//**************************************************************************//
//																			//
// File:		Loader File Access.c										//
//																			//
// Project:		ADSP-2191 Host Port booting									//
//																			//
// Description: This file contains the following functions for accessing	//
//				the loader file:											//
//				   - Open_Loader_File										//
//				   - Close_Loader_File										//
//				   - Read_Next_Loader_Value									//
//																			//
//**************************************************************************//

#include "Loader File Access.h"

// index counter for accessing the data in the loader file
static unsigned int uiLoader_File_Index;

//--------------------------------------------------------------------------//
// Function:	Open_Loader_File											//
//																			//
// Parameters:	None														//
//																			//
// Return:		None														//
//																			//
// Description:	This function opens the loader file.						//
//																			//
// Note:		For the current configuration, with the loader file stored	//
//				in Flash, this function simply resets the read pointer to	//
//				the start address of the loader file in Flash.				//
//				For a different configuration, e.g. loader file stored in a	//
//				physical file, this function should contain the required	//
//				instructions for opening the loader file.					//
//--------------------------------------------------------------------------//
void Open_Loader_File(void)
{
	// reset read pointer to beginning of loader file in Flash
	uiLoader_File_Index = LOADER_FILE_FLASH_ADDRESS;
}

//--------------------------------------------------------------------------//
// Function:	Close_Loader_File											//
//																			//
// Parameters:	None														//
//																			//
// Return:		None														//
//																			//
// Description:	This function closes the loader file.						//
//																			//
// Note:		For the current configuration, with the loader file stored	//
//				in Flash, this function does not do anything.				//
//				For a different configuration, e.g. loader file stored in a	//
//				physical file, this function should contain the required	//
//				instructions for closing the loader file.					//
//--------------------------------------------------------------------------//
void Close_Loader_File(void)
{
	// nothing required for current configuration
}

//--------------------------------------------------------------------------//
// Function:	Read_Next_Loader_Value										//
//																			//
// Parameters:	None														//
//																			//
// Return:		16-bit value from loader file								//
//																			//
// Description:	This function reads the next value from the loader file and	//
//				increments the pointer to the current position,				//
//				uiLoader_File_Index, for every access.						//
//																			//
// Note:		1. Before this function is called the first time, function	//
//				Open_Loader_File should be called.							//
//				2. For the current configuration, with the loader file		//
//				stored in Flash, this function reads two values from the	//
//				8-bit Flash to obtain one 16-bit value from the loader file.//
//--------------------------------------------------------------------------//
unsigned int Read_Next_Loader_Value(void)
{
	unsigned int uiLoader_Value;

	// read next value from loader file and increment index pointer
	uiLoader_Value = external_memory_read(LOADER_FILE_FLASH_PAGE, (void *)uiLoader_File_Index++) << 8;
	uiLoader_Value = uiLoader_Value | external_memory_read(LOADER_FILE_FLASH_PAGE, (void *)uiLoader_File_Index++) & 0x00ff;

	return(uiLoader_Value);
}

