---------------------------------------------------------------------------------------------------
--
-- Name:		External Port state machine module
--
-- Project:		ADSP-2191 Host Port Booting
-- File Name:	ep_sm.vhd
--
---------------------------------------------------------------------------------------------------
--
-- Inputs:		i_clock
--					Clock for External Port state machine
--				i_reset
--					Reset for External Port state machine
--				i_start
--					Start signal for External Port state machine
--				i_finish_ep_access
--					Signal indicating that External Port state machine can be finished
--
-- Outputs:		o_ack
--					EP signal ACK, also used to reset external watchdog
--
-- Description:	This module implements a state machine that generates/expects signals following the
--				timing given in the ADSP-2191 Data Sheet for External Port accesses.
--				The state diagram is given in the application note "ADSP-2191 Host Port Booting".
--
---------------------------------------------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE work.pack_const.ALL;

-- entity for External Port state machine
ENTITY ep_sm IS
	PORT(
		-- control signals for External Port state machine
		i_reset				: IN	STD_LOGIC;	-- asynchronous reset
		i_clock				: IN	STD_LOGIC;	-- system clock
		i_start				: IN	STD_LOGIC;	-- start signal for External Port state machine
		i_finish_ep_access	: IN	STD_LOGIC;	-- EP SM can be finished (generation of ACK)

		-- External Port signals
		o_ack				: OUT	STD_LOGIC	-- ACK and reset for watchdog
	);
END ep_sm;

-- architecture for External Port state machine
ARCHITECTURE ep_sm OF ep_sm IS
	TYPE state_type IS (IDLE,					-- all signals deasserted
						WAIT_HP_ACCESS,
						ACK_ASSERTED);			-- assert ACK
	SIGNAL state : state_type;

BEGIN
	-- determine next state
	next_state_logic: PROCESS(i_reset, i_clock)
	BEGIN
		IF (i_reset = ASSERTED) THEN
			state <= IDLE;
		ELSIF (i_clock'EVENT AND i_clock = ASSERTED) THEN
			CASE state IS
				-- IDLE -> WAIT_HP_ACCESS
				WHEN IDLE =>
					IF (i_start = ASSERTED) THEN
						state <= WAIT_HP_ACCESS;
					END IF;

				-- WAIT_HP_ACCESS -> ACK_ASSERTED
				WHEN WAIT_HP_ACCESS =>
					IF (i_finish_ep_access = ASSERTED) THEN
						state <= ACK_ASSERTED;
					END IF;

				-- ACK_ASSERTED -> IDLE
				WHEN ACK_ASSERTED =>
					state <= IDLE;

				-- undefined states -> IDLE
				WHEN OTHERS =>
					state <= IDLE;
			END CASE;
		END IF;
	END PROCESS next_state_logic;

	-- set output signals
	output_logic: PROCESS(state)
	BEGIN
		CASE state IS
			-- deassert ACK
			WHEN IDLE =>
				o_ack <= EACK_DEASSERTED;

			-- keep ACK deasserted
			WHEN WAIT_HP_ACCESS =>
				o_ack <= EACK_DEASSERTED;

			-- assert ACK
			WHEN ACK_ASSERTED =>
				o_ack <= EACK_ASSERTED;

			-- deassert all output signals in unused states
			WHEN OTHERS =>
				o_ack <= EACK_DEASSERTED;
		END CASE;
	END PROCESS output_logic;
END ep_sm;
