---------------------------------------------------------------------------------------------------
--
-- Name:		Host Port state machine module
--
-- Project:		ADSP-2191 Host Port Booting
-- File Name:	hp_sm.vhd
--
---------------------------------------------------------------------------------------------------
--
-- Inputs:		i_clock
--					Clock for Host Port state machine
--				i_reset
--					Reset for Host Port state machine
--				i_start
--					Start signal for Host Port state machine
--				i_host_cycle
--					Signal indicating current access to Host Port
--						DEASSERTED:	Address cycle (HALE is used)
--						ASSERTED:	Data cycle (HALE is not used)
--				i_host_mem_sel
--					Signal indicating current target for Host Port access
--						DEASSERTED:	Memory space (/HCMS is used)
--						ASSERTED:	IO space (/HCIOMS is used)
--				i_host_strobe
--					Signal indicating read/write access
--						DEASSERTED:	Write access (/HWR is used)
--						ASSERTED:	Read access (/HRD is used)
--				i_hack
--					HP signal HACK
--
-- Outputs:		o_latch_had
--					Signal that is asserted for one clock cycle when data on HAD can be latched
--				o_sm_finished
--					Signal that is asserted for one clock cycle when HP SM transitions to IDLE
--				o_hcms
--					HP signal /HCMS
--				o_hcioms
--					HP signal /HCIOMS
--				o_hale
--					HP signal HALE
--				o_hwr
--					HP signal /HWR
--				o_hrd
--					HP signal /HRD
--
-- Description:	This module implements a state machine that generates/expects signals following the
--				timing given in the ADSP-2191 Data Sheet for Host Port accesses using modes ACK and
--				ACC.
--
---------------------------------------------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE work.pack_const.ALL;

-- entity for Host Port state machine
ENTITY hp_sm IS
	PORT(
		-- control signals for Host Port state machine
		i_reset				: IN	STD_LOGIC;	-- asynchronous reset
		i_clock				: IN	STD_LOGIC;	-- Host Port state machine runs at system clock
		i_start				: IN	STD_LOGIC;	-- start signal for Host Port state machine

		-- configuration signals for state machine
		i_host_cycle		: IN	STD_LOGIC;	-- DEASSERTED: address cycle, ASSERTED: data cycle
		i_host_mem_sel		: IN	STD_LOGIC;	-- DEASSERTED: memory access, ASSERTED: IO access
		i_host_strobe		: IN	STD_LOGIC;	-- DEASSERTED: write access, ASSERTED: read access

		-- status signals from Host Port state machine
		o_latch_had			: OUT	STD_LOGIC;	-- asserted when data on HAD valid
		o_sm_finished		: OUT	STD_LOGIC;	-- asserted when HP SM finished

		-- Host Port signals
		i_hack				: IN	STD_LOGIC;	-- HACK
		o_hcms				: OUT	STD_LOGIC;	-- /HCMS
		o_hcioms			: OUT	STD_LOGIC;	-- /HCIOMS
		o_hale				: OUT	STD_LOGIC;	-- HALE
		o_hwr				: OUT	STD_LOGIC;	-- /HWR
		o_hrd				: OUT	STD_LOGIC	-- /HRD
	);
END hp_sm;

-- architecture for Host Port state machine
ARCHITECTURE hp_sm OF hp_sm IS
	TYPE state_type IS (IDLE,					-- all signals deasserted
						MEM_SEL_ASSERTED,		-- assert /HCMS or /HCIOS
						ADDR_LATCH_ASSERTED,	-- assert HALE
						STROBE_ASSERTED,		-- assert /HWR or /HRD
						WAIT_HACK,				-- wait for HACK
						LATCH_HAD,				-- latch data from HAD[15..0] in read access
						WAIT_LATCH_HAD,
						STROBE_DEASSERTED,		-- deassert /HWR or /HRD
						ADDR_LATCH_DEASSERTED,	-- deassert HALE
						MEM_SEL_DEASSERTED		-- deassert /HCMS or /HCIOS
	);
	SIGNAL state : state_type;

BEGIN
	-- determine next state
	next_state_logic: PROCESS(i_reset, i_clock)
	BEGIN
		IF (i_reset = ASSERTED) THEN
			state <= IDLE;
		ELSIF (i_clock'EVENT AND i_clock = ASSERTED) THEN
			CASE state IS
				-- IDLE -> MEM_SEL_ASSERTED
				WHEN IDLE =>
					IF (i_start = ASSERTED) THEN
						state <= MEM_SEL_ASSERTED;
					END IF;

				-- MEM_SEL_ASSERTED -> STROBE_ASSERTED or ADDR_LATCH_ASSERTED
				WHEN MEM_SEL_ASSERTED =>
					IF (i_host_cycle = ADDRESS_CYCLE) THEN	-- address cycle
						state <= ADDR_LATCH_ASSERTED;
					ELSE						-- data cycle
						state <= STROBE_ASSERTED;
					END IF;

				-- ADDR_LATCH_ASSERTED -> STROBE_ASSERTED
				WHEN ADDR_LATCH_ASSERTED =>
					state <= STROBE_ASSERTED;

				-- STROBE_ASSERTED -> WAIT_HACK
				WHEN STROBE_ASSERTED =>
					state <= WAIT_HACK;

				-- WAIT_HACK -> STROBE_DEASSERTED or LATCH_HAD
				WHEN WAIT_HACK =>
					IF (i_hack = HACK_ASSERTED) THEN
						IF (i_host_strobe = WRITE_ACCESS) THEN	-- write access
							state <= STROBE_DEASSERTED;
						ELSE					-- read access
							state <= LATCH_HAD;
						END IF;
					END IF;

				-- LATCH_HAD -> WAIT_LATCH_HAD
				WHEN LATCH_HAD =>
						state <= WAIT_LATCH_HAD;

				-- WAIT_LATCH_HAD -> STROBE_DEASSERTED
				WHEN WAIT_LATCH_HAD =>
						state <= STROBE_DEASSERTED;

				-- STROBE_DEASSERTED -> MEM_SEL_DEASSERTED or ADDR_LATCH_DEASSERTED
				WHEN STROBE_DEASSERTED =>
					IF (i_host_cycle = ADDRESS_CYCLE) THEN	-- address cycle
						state <= ADDR_LATCH_DEASSERTED;
					ELSE						-- data cycle
						state <= MEM_SEL_DEASSERTED;
					END IF;

				-- ADDR_LATCH_DEASSERTED -> MEM_SEL_DEASSERTED
				WHEN ADDR_LATCH_DEASSERTED =>
					state <= MEM_SEL_DEASSERTED;

				-- MEM_SEL_DEASSERTED -> IDLE
				WHEN MEM_SEL_DEASSERTED =>
					state <= IDLE;

				-- undefined states -> IDLE
				WHEN OTHERS =>
					state <= IDLE;
			END CASE;
		END IF;
	END PROCESS next_state_logic;

	-- assign output signals for all states
	output_logic: PROCESS(state)
	BEGIN
		CASE state IS
			-- deassert all signals
			WHEN IDLE =>
				o_hcms			<= HCMS_DEASSERTED;
				o_hcioms		<= HCIOMS_DEASSERTED;
				o_hale			<= HALE_DEASSERTED;
				o_hwr			<= HWR_DEASSERTED;
				o_hrd			<= HRD_DEASSERTED;
				o_latch_had		<= DEASSERTED;
				o_sm_finished	<= DEASSERTED;

			-- assert /HCMS or /HCIOMS depending on accessed memory space
			WHEN MEM_SEL_ASSERTED =>
				IF (i_host_mem_sel = MEM_SPACE) THEN	-- Memory space access
					o_hcms	<= HCMS_ASSERTED;
				ELSE							-- IO space access
					o_hcioms	<= HCIOMS_ASSERTED;
				END IF;

			-- assert HALE
			WHEN ADDR_LATCH_ASSERTED =>
				o_hale <= HALE_ASSERTED;

			-- assert /HWR or /HRD depending on read or write access
			WHEN STROBE_ASSERTED =>
				IF (i_host_strobe = WRITE_ACCESS) THEN	-- write access
					o_hwr <= HWR_ASSERTED;
				ELSE							-- read access
					o_hrd <= HRD_ASSERTED;
				END IF;

			-- no output change
			WHEN WAIT_HACK =>
				NULL;

			-- data from HAD can be latched on higher level
			WHEN LATCH_HAD =>
				o_latch_had <= ASSERTED;

			-- deassert latch signal for HAD
			WHEN WAIT_LATCH_HAD =>
				o_latch_had	<= DEASSERTED;

			-- deassert /HWR or /HRD (only one was asserted, depending on read or write access);
			WHEN STROBE_DEASSERTED =>
				o_hrd		<= HRD_DEASSERTED;
				o_hwr		<= HWR_DEASSERTED;

			-- deassert HALE
			WHEN ADDR_LATCH_DEASSERTED =>
				o_hale <= HALE_DEASSERTED;

			-- deassert /HCIOMS or /HCMS (only one was asserted, depending on accessed memory space)
			WHEN MEM_SEL_DEASSERTED =>
				o_hcioms		<= HCIOMS_DEASSERTED;
				o_hcms			<= HCMS_DEASSERTED;
				o_sm_finished	<= ASSERTED;

			-- deassert all output signals in unused states
			WHEN OTHERS =>
				o_hcms			<= HCMS_DEASSERTED;
				o_hcioms		<= HCIOMS_DEASSERTED;
				o_hale			<= HALE_DEASSERTED;
				o_hwr			<= HWR_DEASSERTED;
				o_hrd			<= HRD_DEASSERTED;
				o_latch_had		<= DEASSERTED;
				o_sm_finished	<= DEASSERTED;
		END CASE;
	END PROCESS output_logic;
END hp_sm;
